# Standard
import sys

# Python2/3 compat
if sys.version_info[0] == 3:
    text_type = str
    binary_type = bytes
    string_types = (str,)
else:
    text_type = unicode
    binary_type = str
    string_types = (basestring,)



# Standard
import logging

# Project
from log_color import colors
from log_color.regex import COLOR_EXP
from log_color.compat import text_type


class ColorStripper(logging.Formatter):
    """
    Strip references to colored output from message.

    This formatter removes all color code text from a message. For example
    it would convert ``this #g<is> text`` to ``this is text``.
    """

    def format(self, record):
        """Format the message."""
        msg = logging.Formatter.format(self, record)
        msg = colors.strip_color(msg)
        for val in COLOR_EXP.findall(msg):
            if val.startswith("#d"):
                msg = msg.replace(val, val[4:-1])
            else:
                msg = msg.replace(val, val[3:-1])
        return msg


class ColorFormatter(logging.Formatter):
    """
    Insert color sequences for logging messages.

    Convert all portions of a message that contain color codes into a ColorStr
    instance that matches the color map data then re-insert it into the
    message. On supported platforms this will insert a color codes around the
    specified text.

    Color sequences should be in the following format: #y<message> which will
    be formatted as yellow. The colors allowed are:

    - m (magenta)
    - b (blue)
    - c (cyan)
    - g (green)
    - y (yellow)
    - r (red)
    - w (white)

    - dm (dark magenta)
    - db (dark blue)
    - dc (dark cyan)
    - dg (dark green)
    - dy (dark yellow)
    - dr (dark red)
    """

    def format(self, record):
        """Format the message."""
        msg = logging.Formatter.format(self, record)
        for val in COLOR_EXP.findall(msg):
            if val.startswith("#d"):
                msg = msg.replace(val, colors.ColorStr(val[4:-1], colors.COLOR_MAP[val[1:3]]))
            else:
                msg = msg.replace(val, colors.ColorStr(val[3:-1], colors.COLOR_MAP[val[1]]))
        return text_type(msg)



from setuptools import setup, find_packages

long_description = (
    "Logging formatters for colorized outputs. A simple domain specific "
    "language for log messages which can be used to colorize all or part of "
    "a particular log message. For example:\n\n"
    "LOG.debug('Found file at #g<%s>', path)\n\n"
    "would colorize the contents of the 'path' variable green when output to "
    "a command line terminal.\n\nA formatter which strips color sequences "
    "from the output is also included for situations like logging to files "
    "where it would not make sense to have ANSI color sequences embedded in "
    "the output."
)

setup(
    name="log_color",
    author="Brant Watson",
    author_email="oldspiceap@gmail.com",
    description="Simple log formatters for colored output",
    long_description=long_description,
    version="1.0.7",
    url="https://github.com/induane/logcolor",
    license="wtfpl",
    packages=find_packages("src"),
    package_dir={"": "src"},
    python_requires=">2.7, !=3.0.*, !=3.1.*, !=3.2.*",
)



# Standard
import os
import sys
import platform

# Project
from log_color import compat

COLOR_MAP = {
    "b": u"\033[94m",
    "c": u"\033[96m",
    "g": u"\033[92m",
    "m": u"\033[95m",
    "r": u"\033[91m",
    "y": u"\033[93m",
    "w": u"\033[97m",
    "db": u"\033[34m",
    "dc": u"\033[36m",
    "dg": u"\033[32m",
    "dm": u"\033[35m",
    "dr": u"\033[31m",
    "dy": u"\033[33m",
}

COLOR_END = "\033[0m"
# Assemble list of all color sequences
ALL_SEQ = frozenset(
    [x for x in COLOR_MAP.values()]
    + [
        COLOR_END,
    ]
)


def strip_color(value):
    """Strip all color values from a given string"""
    for seq in ALL_SEQ:
        value = value.replace(seq, "")
    return value


class ColorStr(compat.text_type):
    """Subclasses string to optionally include ascii color"""

    def __init__(self, *args, **kwargs):
        super(ColorStr, self).__init__()

    def __new__(cls, value, color, force_seq=None, *args, **kwargs):
        if cls.color_supported(force_seq=force_seq):
            return compat.text_type.__new__(cls, u"{0}{1}{2}".format(color, value, COLOR_END))
        return compat.text_type.__new__(cls, value)

    @staticmethod
    def color_supported(force_seq=None):
        """Shoddy detection of color support."""
        # If True or False, override autodetection and return.
        if force_seq is False or force_seq is True:
            return force_seq

        # Honor NO_COLOR environment variable:
        if os.environ.get("NO_COLOR", None):
            return False

        # Attempt simple autodetection
        if (hasattr(sys.stderr, "isatty") and sys.stderr.isatty()) or (
            "TERM" in os.environ and os.environ["TERM"] == "ANSI"
        ):
            if "windows" in platform.system().lower() and not ("TERM" in os.environ and os.environ["TERM"] == "ANSI"):
                return False  # Windows console, no ANSI support
            else:
                return True  # ANSI output allowed
        else:
            return False  # ANSI output not allowed
